@@ -10,6 +10,7 @@ from estime2.helper import (
     raise_if_not_subset,
     return_op_if_None
 )
+from estime2.provpoptable_resultswrapper import ProvPopTableResultsWrapper
 from typing import Collection as Axes
 from typing import (
     List,
@@ -48,11 +49,15 @@ class ProvPopTable(pd.DataFrame):
         comp_neg_npr_out: Optional[str] = None,
         comp_neg_death: Optional[str] = None,
         comp_neg_interprov_out: Optional[str] = None,
+        comp_neg_etc: Optional[List[str]] = None,
+        comp_neg_put_etc_before: Optional[bool] = None,
         comp_pos_temp_in: Optional[str] = None,
         comp_pos_ret_emi: Optional[str] = None,
         comp_pos_npr_in: Optional[str] = None,
         comp_pos_immi: Optional[str] = None,
         comp_pos_interprov_in: Optional[str] = None,
+        comp_pos_etc: Optional[List[str]] = None,
+        comp_pos_put_etc_before: Optional[bool] = None,
         comp_end: Optional[List[str]] = None,
         reorder_cols: bool = True,
         show_pop_end: bool = False,
@@ -81,11 +86,15 @@ class ProvPopTable(pd.DataFrame):
             comp_neg_npr_out: Optional[str] = None,
             comp_neg_death: Optional[str] = None,
             comp_neg_interprov_out: Optional[str] = None,
+            comp_neg_etc: Optional[List[str]] = None,
+            comp_neg_put_etc_before: Optional[bool] = None,
             comp_pos_temp_in: Optional[str] = None,
             comp_pos_ret_emi: Optional[str] = None,
             comp_pos_npr_in: Optional[str] = None,
             comp_pos_immi: Optional[str] = None,
             comp_pos_interprov_in: Optional[str] = None,
+            comp_pos_etc: Optional[List[str]] = None,
+            comp_pos_put_etc_before: Optional[bool] = None,
             comp_end: Optional[List[str]] = None,
             reorder_cols: bool = True,
             show_pop_end: bool = False,
@@ -191,6 +200,25 @@ class ProvPopTable(pd.DataFrame):
             having the same name as `comp_neg.interprov_out` is selected as
             the "Interprovincial migrant OUT" column. If not, the method 
             raises `AssertionError`.
+        * `comp_neg_etc`: (`None` by default) a list of str; the names of
+            columns corresponding to the remaining negative components
+            in the population table. If `None`, it first checks whether the
+            global option value `comp_neg.etc` is `[]`, an empty list. If
+            it is an empty list, then it assumes no extra negative 
+            component is given in the table, and therefore not used. If it
+            is not `None`, the method then checks whether the value of
+            `comp_neg.etc` itself is a subset of all the column names in
+            the population table. If it is, the column(s) in `comp_neg.etc`
+            is used in the computation of end-of-period populations as well
+            as in the correction process. If not, the method raises 
+            `AssertionError`.
+        * `comp_neg_put_etc_before`: (`None` by default) a bool; if `True`,
+            then components in `comp_neg.etc` will precede 
+            `comp_neg.interprov_out` and follow other conventional negative
+            components. If `False`, components will follow 
+            `comp_neg.interprov_out` instead. If `None`, then it uses a 
+            global option value `estime2.options.comp_neg.put_etc_before` 
+            (`False` by default).
         * `comp_pos_temp_in`: (`None` by default) a str; the name of
             the column corresponding to "Temporary emigrants IN" in the
             population table. If `None`, it first checks whether the 
@@ -249,6 +277,25 @@ class ProvPopTable(pd.DataFrame):
             as `comp_pos.interprov_in` is selected as the 
             "Interprovincial migrant IN" column. If not, the method raises
             `AssertionError`.
+        * `comp_pos_etc`: (`None` by default) a list of str; the names of
+            columns corresponding to the remaining positive components
+            in the population table. If `None`, it first checks whether the
+            global option value `comp_pos.etc` is `[]`, an empty list. If
+            it is an empty list, then it assumes no extra positive 
+            component is given in the table, and therefore not used. If it
+            is not `None`, the method then checks whether the value of
+            `comp_pos.etc` itself is a subset of all the column names in
+            the population table. If it is, the column(s) in `comp_pos.etc`
+            is used in the computation of end-of-period populations as well
+            as in the correction process. If not, the method raises 
+            `AssertionError`.
+        * `comp_pos_put_etc_before`: (`None` by default) a bool; if `True`,
+            then components in `comp_pos.etc` will precede 
+            `comp_pos.interprov_in` and follow other conventional positive
+            components. If `False`, components will follow 
+            `comp_pos.interprov_in` instead. If `None`, then it uses a 
+            global option value `estime2.options.comp_pos.put_etc_before` 
+            (`False` by default).
         * `comp_end`: (`None` by default) a list of str; component(s) that
             is/are recorded at the end of the period. If `None`, then
             the global option value `comp.end` is used.
@@ -258,7 +305,7 @@ class ProvPopTable(pd.DataFrame):
         * `show_pop_end`: (`False` by default) a boolean; if True, leave
             end-of-period population in a ProvPopTable, and discard it
             otherwise.
-        * `flag`: (`True` by default) a boolean; leave this argument to be
+        * `flag`: (`True` by default) a boolean; leave this argument as
             `True`. This argument is for stopping the recursion in
             `ProvPopTable.__init__()`.
         
@@ -277,18 +324,20 @@ class ProvPopTable(pd.DataFrame):
             pattern `pop_*` cannot have `None` as its value. For example,
             `estime2.options.pop.age = None` raises an error.
         6. Any global option corresponding to the arguments with the
-            pattern `comp_*` may have `None` as its value. In case where
-            both the argument and the corresponding global option are 
-            `None`, the corresponding column is discarded from the table
-            (i.e. not shown in the table).
+            pattern `comp_*` except `comp_end` and `comp_*.*etc*` may have
+            `None` as its value. In case where both the argument and the 
+            corresponding global option are `None`, the corresponding 
+            column is discarded from the table (i.e. not shown and not 
+            present in the table).
         7. Argument values have precedence over global option values. That
             is, if the argument value and the corresponding global option 
-            value are different, the method uses the argument value.
+            value are different, this `__init__` method uses the argument 
+            value.
         8. If `reorder_cols` is `True`, then the columns of `data` is
             reordered in the following order from the left. For a column
             corresponding to one of the `comp_*` arguments, it will be
             displayed in the table iff neither the argument nor the 
-            corresponding global option is `None`: 
+            corresponding global option is `None` or `[]`: 
                 + `pop_sex`
                 + `pop_age`
                 + `pop_end` (if `show_pop_end` is True)
@@ -298,22 +347,25 @@ class ProvPopTable(pd.DataFrame):
                 + `comp_neg_emi`
                 + `comp_neg_npr_out`
                 + `comp_neg_death`
+                + `comp_neg_etc` (if given any & put_etc_before is True)
                 + `comp_neg_interprov_out`
+                + `comp_neg_etc` (if given any & put_etc_before is False)
                 + `comp_pos_temp_in`
                 + `comp_pos_ret_emi`
                 + `comp_pos_npr_in`
                 + `comp_pos_immi`
+                + `comp_pos_etc` (if given any & put_etc_before is True)
                 + `comp_pos_interprov_in`
-        9. Whether `reorder_cols` is True or False, the method looks for
+                + `comp_pos_etc` (if given any & put_etc_before is False)
+        9. Whether or not `reorder_cols` is `True`, the method looks for
             the component(s) responsible for the negative end-of-period
-            population following the above order.
+            population following the above order of `comp_*`'s.
         '''
 
         # Inherit from pandas.DataFrame
         super().__init__(data, index, columns, dtype, copy)
         self.fillna(0, inplace = True)
 
-        # **kwargs required
         opname_arg = {
             'pop.sex': pop_sex,
             'pop.age': pop_age,
@@ -325,11 +377,15 @@ class ProvPopTable(pd.DataFrame):
             'comp_neg.npr_out': comp_neg_npr_out,
             'comp_neg.death': comp_neg_death, 
             'comp_neg.interprov_out': comp_neg_interprov_out,
+            'comp_neg.etc': comp_neg_etc,
+            'comp_neg.put_etc_before': comp_neg_put_etc_before,
             'comp_pos.temp_in': comp_pos_temp_in, 
             'comp_pos.ret_emi': comp_pos_ret_emi, 
             'comp_pos.npr_in': comp_pos_npr_in,
             'comp_pos.immi': comp_pos_immi, 
             'comp_pos.interprov_in': comp_pos_interprov_in,
+            'comp_pos.etc': comp_pos_etc,
+            'comp_pos.put_etc_before': comp_pos_put_etc_before,
             'comp.end': comp_end
         }
         for opname, arg in opname_arg.copy().items():
@@ -348,13 +404,17 @@ class ProvPopTable(pd.DataFrame):
                         opname_arg['comp_neg.emi'],
                         opname_arg['comp_neg.npr_out'],
                         opname_arg['comp_neg.death'],
-                        opname_arg['comp_neg.interprov_out'],
+                        opname_arg['comp_neg.interprov_out']
+                    ] +\
+                    opname_arg['comp_neg.etc'] +\
+                    [
                         opname_arg['comp_pos.temp_in'],
                         opname_arg['comp_pos.ret_emi'],
                         opname_arg['comp_pos.npr_in'],
                         opname_arg['comp_pos.immi'],
                         opname_arg['comp_pos.interprov_in']
-                    ],
+                    ] +\
+                    opname_arg['comp_pos.etc'],
                     'comp_end',
                     'components'
                 )
@@ -370,7 +430,20 @@ class ProvPopTable(pd.DataFrame):
             if opname3 == 'pop.end':
                 continue
             if arg3 is not None:
-                if opname3 != 'comp.end':
+                # Note: the only way arg3 is None at this point is when 
+                # both the provided argument as well as the corresponding 
+                # global option are None.
+                # If the validator of the corresponding global option 
+                # does NOT allow NoneType, then opname3 is checked for
+                # whether it is a subset of all_cols.
+                if opname3 in [
+                    'comp_neg.put_etc_before',
+                    'comp_pos.put_etc_before'
+                ]:
+                    continue
+                elif opname3 not in [
+                    'comp.end', 'comp_neg.etc', 'comp_pos.etc'
+                ]:
                     raise_if_not_subset(
                         [arg3], all_cols, 
                         opname3.replace('.', '_'), all_name
@@ -392,15 +465,33 @@ class ProvPopTable(pd.DataFrame):
             reordered_cols2.append(opname_arg['pop.end'])
         reordered_cols2.append(opname_arg['pop.start'])
         reordered_cols2.append(opname_arg['pop.birth'])
-        comp_ordered = [
+
+        comp_negs = [
             'comp_neg.temp_out', 'comp_neg.emi', 'comp_neg.npr_out',
-            'comp_neg.death', 'comp_neg.interprov_out',
-            'comp_pos.temp_in', 'comp_pos.ret_emi', 'comp_pos.npr_in',
-            'comp_pos.immi', 'comp_pos.interprov_in'
+            'comp_neg.death'
         ]
+        if opname_arg['comp_neg.put_etc_before']:
+            comp_negs.append('comp_neg.etc')
+        comp_negs.append('comp_neg.interprov_out')
+        if not opname_arg['comp_neg.put_etc_before']:
+            comp_negs.append('comp_neg.etc')
+        comp_poss = [
+            'comp_pos.temp_in', 'comp_pos.ret_emi', 'comp_pos.npr_in',
+            'comp_pos.immi'
+        ]
+        if opname_arg['comp_pos.put_etc_before']:
+            comp_poss.append('comp_pos.etc')
+        comp_poss.append('comp_pos.interprov_in')
+        if not opname_arg['comp_pos.put_etc_before']:
+            comp_poss.append('comp_pos.etc')
+        comp_ordered = comp_negs + comp_poss
+
         for comp in comp_ordered:
             if opname_arg[comp] is not None:
-                reordered_cols2.append(opname_arg[comp])
+                if isinstance(opname_arg[comp], list):
+                    reordered_cols2.extend(opname_arg[comp])
+                else:
+                    reordered_cols2.append(opname_arg[comp])
         reordered_cols = reordered_cols1 + reordered_cols2
 
         self[opname_arg['pop.age']] =\
@@ -420,11 +511,15 @@ class ProvPopTable(pd.DataFrame):
         self.__comp_neg_npr_out = opname_arg['comp_neg.npr_out']
         self.__comp_neg_death = opname_arg['comp_neg.death']
         self.__comp_neg_interprov_out = opname_arg['comp_neg.interprov_out']
+        self.__comp_neg_etc = opname_arg['comp_neg.etc']
+        self.__comp_neg_put_etc_before = opname_arg['comp_neg.put_etc_before']
         self.__comp_pos_temp_in = opname_arg['comp_pos.temp_in']
         self.__comp_pos_ret_emi = opname_arg['comp_pos.ret_emi']
         self.__comp_pos_npr_in = opname_arg['comp_pos.npr_in']
         self.__comp_pos_immi = opname_arg['comp_pos.immi']
         self.__comp_pos_interprov_in = opname_arg['comp_pos.interprov_in']
+        self.__comp_pos_etc = opname_arg['comp_pos.etc']
+        self.__comp_pos_put_etc_before = opname_arg['comp_pos.put_etc_before']
         self.__comp_end = opname_arg['comp.end']
 
         if reorder_cols:
@@ -443,11 +538,15 @@ class ProvPopTable(pd.DataFrame):
                 comp_neg_npr_out = opname_arg['comp_neg.npr_out'],
                 comp_neg_death = opname_arg['comp_neg.death'],
                 comp_neg_interprov_out = opname_arg['comp_neg.interprov_out'],
+                comp_neg_etc = opname_arg['comp_neg.etc'],
+                comp_neg_put_etc_before = opname_arg['comp_neg.put_etc_before'],
                 comp_pos_temp_in = opname_arg['comp_pos.temp_in'],
                 comp_pos_ret_emi = opname_arg['comp_pos.ret_emi'],
                 comp_pos_npr_in = opname_arg['comp_pos.npr_in'],
                 comp_pos_immi = opname_arg['comp_pos.immi'],
                 comp_pos_interprov_in = opname_arg['comp_pos.interprov_in'],
+                comp_pos_etc = opname_arg['comp_pos.etc'],
+                comp_pos_put_etc_before = opname_arg['comp_pos.put_etc_before'],
                 comp_end = opname_arg['comp.end'],
                 reorder_cols = False,
                 show_pop_end = show_pop_end,
@@ -478,11 +577,15 @@ class ProvPopTable(pd.DataFrame):
                 comp_neg_npr_out = opname_arg['comp_neg.npr_out'],
                 comp_neg_death = opname_arg['comp_neg.death'],
                 comp_neg_interprov_out = opname_arg['comp_neg.interprov_out'],
+                comp_neg_etc = opname_arg['comp_neg.etc'],
+                comp_neg_put_etc_before = opname_arg['comp_neg.put_etc_before'],
                 comp_pos_temp_in = opname_arg['comp_pos.temp_in'],
                 comp_pos_ret_emi = opname_arg['comp_pos.ret_emi'],
                 comp_pos_npr_in = opname_arg['comp_pos.npr_in'],
                 comp_pos_immi = opname_arg['comp_pos.immi'],
                 comp_pos_interprov_in = opname_arg['comp_pos.interprov_in'],
+                comp_pos_etc = opname_arg['comp_pos.etc'],
+                comp_pos_put_etc_before = opname_arg['comp_pos.put_etc_before'],
                 comp_end = opname_arg['comp.end'],
                 reorder_cols = False,
                 show_pop_end = show_pop_end,
@@ -517,12 +620,16 @@ class ProvPopTable(pd.DataFrame):
             '_ProvPopTable__comp_neg_emi', 
             '_ProvPopTable__comp_neg_npr_out', 
             '_ProvPopTable__comp_neg_death', 
-            '_ProvPopTable__comp_neg_interprov_out', 
+            '_ProvPopTable__comp_neg_interprov_out',
+            '_ProvPopTable__comp_neg_etc', 
+            '_ProvPopTable__comp_neg_put_etc_before',
             '_ProvPopTable__comp_pos_temp_in',
             '_ProvPopTable__comp_pos_ret_emi',
             '_ProvPopTable__comp_pos_npr_in',
             '_ProvPopTable__comp_pos_immi',
             '_ProvPopTable__comp_pos_interprov_in',
+            '_ProvPopTable__comp_pos_etc',
+            '_ProvPopTable__comp_pos_put_etc_before',
             '_ProvPopTable__comp_end'
         ]
         if name in allowed_attrs:
@@ -581,6 +688,8 @@ class ProvPopTable(pd.DataFrame):
             'comp_pos', 'positive components'
         )
 
+        pop_end = self.__pop_end
+        pop_groups = self.get_pop_groups()
         comps = comp_neg + comp_pos
         comp_end = self.__comp_end
         comp_not_end = []
@@ -588,7 +697,7 @@ class ProvPopTable(pd.DataFrame):
             for comp in comps:
                 if comp not in comp_end:
                     comp_not_end.append(comp)
-        pop_groups = [self.__pop_sex, self.__pop_age]
+        
         comp_aggs = {}
         comp_aggs[self.__pop_start] = 'sum'
         comp_aggs[self.__pop_birth] = 'sum'
@@ -616,13 +725,15 @@ class ProvPopTable(pd.DataFrame):
         result2 = self.loc[:, pop_groups + comp_end]
         result2 = result2.loc[lambda df: df[self.__pop_age] != -1]
 
+        result1[self.__pop_age] = result1[self.__pop_age].apply(str)
+        result2[self.__pop_age] = result2[self.__pop_age].apply(str)
         result3 = result1\
             .merge(
                 result2,
                 how = 'left',
                 on = pop_groups
             )
-        pop_end = self.__pop_end
+        result3[self.__pop_age] = result3[self.__pop_age].apply(Age)
         result3[pop_end] =\
             result3[self.__pop_start] + result3[self.__pop_birth]
         if comp_neg != []:
@@ -675,74 +786,36 @@ class ProvPopTable(pd.DataFrame):
         --------
         * `method`: the same argument as in `self.get_K(comp, method, J)`.
         * `return_all_mods`: a bool (`False` by default); if `True`, this 
-            will return all the modifications and counter-adjustments made.
+            will return all the modifications and counter-adjustments made
+            to `self`.
 
         Returns
         -------
-        `ProvPopTable` if interprovincial components are NOT modified AND
-        `return_all_mods` is `False`; a `list` of `ProvPopTable` and 
-        `pandas.DataFrame` if either interprovincial components are 
-        modified, or `return_all_mods` is `True`.
+        If `return_all_mods` is `False`, then it returns a fixed version
+        of `self`, the current `ProvPopTable`. 
+        If `return_all_mods` is `True` and there are NO corrections made
+        to the original table, then it returns `self`, the current 
+        `ProvPopTable`.
+        If `return_all_mods` is `True` and there are ANY corrections made
+        to the original table, then it returns a 
+        `ProvPopTableResultsWrapper`.
         '''
 
         comp_neg_to_use = self.get_comp_neg()
         comp_pos_to_use = self.get_comp_pos()
         comps = comp_neg_to_use + comp_pos_to_use
 
-        pop_groups = [self.__pop_sex, self.__pop_age]
+        pop_groups = self.get_pop_groups()
         all_cols = self.columns.tolist()
         show_pop_end = self.__pop_end in all_cols
-        interprovs = [
-            self.__comp_neg_interprov_out, 
-            self.__comp_pos_interprov_in
-        ]
         comps_modified = []
-        self_copy = self.copy()
-        self_copy = ProvPopTable(
-            self_copy,
-            pop_sex = self.__pop_sex,
-            pop_age = self.__pop_age,
-            pop_end = self.__pop_end,
-            pop_start = self.__pop_start,
-            pop_birth = self.__pop_birth,
-            comp_neg_temp_out = self.__comp_neg_temp_out,
-            comp_neg_emi = self.__comp_neg_emi,
-            comp_neg_npr_out = self.__comp_neg_npr_out,
-            comp_neg_death = self.__comp_neg_death,
-            comp_neg_interprov_out = self.__comp_neg_interprov_out,
-            comp_pos_temp_in = self.__comp_pos_temp_in,
-            comp_pos_ret_emi = self.__comp_pos_ret_emi,
-            comp_pos_npr_in = self.__comp_pos_npr_in,
-            comp_pos_immi = self.__comp_pos_immi,
-            comp_pos_interprov_in = self.__comp_pos_interprov_in,
-            comp_end = self.__comp_end,
-            reorder_cols = False, # since it is reordered already
-            show_pop_end = show_pop_end,
-            # since records are grouped + summarized by 
-            # sex and age already
-            flag = False
-        )
-
-        flag = True
-        i = 0
-        num_zeros = sum(self_copy.get_I()['I'].values == 0)
-        while flag:
-            comp = comps[i]
-            comp_L = '{0}_L'.format(comp)
-            L = self_copy.get_L(comp, method)
-            if return_all_mods or (comp in interprovs):
-                if any(L[comp_L].values != 0):
-                    comps_modified.append(L)
-            L['Age'] = L['Age'].apply(str)
-            self_copy['Age'] = self_copy['Age'].apply(str)
-            self_copy = self_copy.merge(L, on = pop_groups, how = 'left')
-            self_copy.fillna(0, inplace = True)
-            self_copy[comp_L] = self_copy[comp_L].apply(int)
-            self_copy[comp] += self_copy[comp_L]
-            del self_copy[comp_L]
-
-            self_copy = ProvPopTable(
-                self_copy,
+        sexes = np.unique(self[self.__pop_sex])
+        self_copy = None
+        for sex in sexes:
+            self_cp_sex =\
+                self.copy().loc[lambda df: df[self.__pop_sex] == sex]
+            self_cp_sex = ProvPopTable(
+                self_cp_sex,
                 pop_sex = self.__pop_sex,
                 pop_age = self.__pop_age,
                 pop_end = self.__pop_end,
@@ -753,11 +826,15 @@ class ProvPopTable(pd.DataFrame):
                 comp_neg_npr_out = self.__comp_neg_npr_out,
                 comp_neg_death = self.__comp_neg_death,
                 comp_neg_interprov_out = self.__comp_neg_interprov_out,
+                comp_neg_etc = self.__comp_neg_etc,
+                comp_neg_put_etc_before = self.__comp_neg_put_etc_before,
                 comp_pos_temp_in = self.__comp_pos_temp_in,
                 comp_pos_ret_emi = self.__comp_pos_ret_emi,
                 comp_pos_npr_in = self.__comp_pos_npr_in,
                 comp_pos_immi = self.__comp_pos_immi,
                 comp_pos_interprov_in = self.__comp_pos_interprov_in,
+                comp_pos_etc = self.__comp_pos_etc,
+                comp_pos_put_etc_before = self.__comp_pos_put_etc_before,
                 comp_end = self.__comp_end,
                 reorder_cols = False, # since it is reordered already
                 show_pop_end = show_pop_end,
@@ -765,29 +842,123 @@ class ProvPopTable(pd.DataFrame):
                 # sex and age already
                 flag = False
             )
+            flag = True
+            i = 0
+            num_zeros = sum(self_cp_sex.get_I()['I'].values == 0)
+            while flag:
+                comp = comps[i]
+                comp_L = f"{comp}_L"
+                L = self_cp_sex.get_L(comp, method)
+                if return_all_mods:
+                    if any(L[comp_L].values != 0):
+                        comps_modified.append(L)
+                L[self.__pop_age] =\
+                    L[self.__pop_age].apply(str)
+                self_cp_sex[self.__pop_age] =\
+                    self_cp_sex[self.__pop_age].apply(str)
+                self_cp_sex = self_cp_sex\
+                    .merge(L, on = pop_groups, how = 'left')
+                self_cp_sex.fillna(0, inplace = True)
+                self_cp_sex[comp_L] = self_cp_sex[comp_L].apply(int)
+                self_cp_sex[comp] += self_cp_sex[comp_L]
+                del self_cp_sex[comp_L]
 
-            I_vec = self_copy.get_I()['I'].values
-            new_num_zeros = sum(I_vec == 0)
-            if all(I_vec == 0): # all problematic records fixed
-                flag = False # stop the iteration
-            elif num_zeros != new_num_zeros: # one problematic record fixed
-                i = 0 # move to the next problematic record
-                num_zeros = new_num_zeros # assign a new number
-            elif i != (len(comps) - 1): # correction in progress
-                i += 1
-            else: # i.e. lack of values for corrections
-                # Correction could not be made to the problematic record
-                # at the youngest age.
-                # If the youngest problematic record cannot be fixed, then
-                # any older problematic records cannot be fixed either.
-                # intraprov_in or intraprov_out should be modified 
-                # in SubProvPopTable
-                flag = False
+                self_cp_sex = ProvPopTable(
+                    self_cp_sex,
+                    pop_sex = self.__pop_sex,
+                    pop_age = self.__pop_age,
+                    pop_end = self.__pop_end,
+                    pop_start = self.__pop_start,
+                    pop_birth = self.__pop_birth,
+                    comp_neg_temp_out = self.__comp_neg_temp_out,
+                    comp_neg_emi = self.__comp_neg_emi,
+                    comp_neg_npr_out = self.__comp_neg_npr_out,
+                    comp_neg_death = self.__comp_neg_death,
+                    comp_neg_interprov_out = self.__comp_neg_interprov_out,
+                    comp_neg_etc = self.__comp_neg_etc,
+                    comp_neg_put_etc_before = self.__comp_neg_put_etc_before,
+                    comp_pos_temp_in = self.__comp_pos_temp_in,
+                    comp_pos_ret_emi = self.__comp_pos_ret_emi,
+                    comp_pos_npr_in = self.__comp_pos_npr_in,
+                    comp_pos_immi = self.__comp_pos_immi,
+                    comp_pos_interprov_in = self.__comp_pos_interprov_in,
+                    comp_pos_etc = self.__comp_pos_etc,
+                    comp_pos_put_etc_before = self.__comp_pos_put_etc_before,
+                    comp_end = self.__comp_end,
+                    reorder_cols = False, # since it is reordered already
+                    show_pop_end = show_pop_end,
+                    # since records are grouped + summarized by 
+                    # sex and age already
+                    flag = False                
+                )
 
-        if comps_modified == []:
-            return self_copy
+                I_vec = self_cp_sex.get_I()['I'].values
+                new_num_zeros = sum(I_vec == 0)
+                if all(I_vec == 0):
+                    # all problematic records fixed
+                    flag = False # stop the while-loop iteration
+                elif num_zeros != new_num_zeros:
+                    # one problematic record fixed
+                    i = 0 # move to the next problematic record
+                    num_zeros = new_num_zeros # assign a new number
+                elif i != (len(comps) - 1): 
+                    # correction in progress
+                    i += 1
+                else: 
+                    # i.e. lack of values for corrections
+                    # Correction could not be made to the problematic 
+                    # record at the youngest age.
+                    # If the youngest problematic record cannot be fixed, 
+                    # then any older problematic records cannot be fixed 
+                    # either.
+                    # intraprov_in or intraprov_out should be modified 
+                    # in SubProvPopTable
+                    flag = False
+
+            if self_copy is None:
+                self_copy = self_cp_sex
+            else:
+                self_copy = self_copy\
+                    .append(self_cp_sex, ignore_index = True)
+
+        self_copy = ProvPopTable(
+            self_copy.sort_values(pop_groups),
+            pop_sex = self.__pop_sex,
+            pop_age = self.__pop_age,
+            pop_end = self.__pop_end,
+            pop_start = self.__pop_start,
+            pop_birth = self.__pop_birth,
+            comp_neg_temp_out = self.__comp_neg_temp_out,
+            comp_neg_emi = self.__comp_neg_emi,
+            comp_neg_npr_out = self.__comp_neg_npr_out,
+            comp_neg_death = self.__comp_neg_death,
+            comp_neg_interprov_out = self.__comp_neg_interprov_out,
+            comp_neg_etc = self.__comp_neg_etc,
+            comp_neg_put_etc_before = self.__comp_neg_put_etc_before,
+            comp_pos_temp_in = self.__comp_pos_temp_in,
+            comp_pos_ret_emi = self.__comp_pos_ret_emi,
+            comp_pos_npr_in = self.__comp_pos_npr_in,
+            comp_pos_immi = self.__comp_pos_immi,
+            comp_pos_interprov_in = self.__comp_pos_interprov_in,
+            comp_pos_etc = self.__comp_pos_etc,
+            comp_pos_put_etc_before = self.__comp_pos_put_etc_before,
+            comp_end = self.__comp_end,
+            reorder_cols = False, # since it is reordered already
+            show_pop_end = show_pop_end,
+            # since records are grouped + summarized by 
+            # sex and age already
+            flag = False            
+        )
+        if return_all_mods:
+            if comps_modified == []:
+                return self_copy
+            result = ProvPopTableResultsWrapper(
+                orig_table = self, 
+                result_fix_issues = [self_copy] + comps_modified
+            )
+            return result
         else:
-            return [self_copy] + comps_modified
+            return self_copy
 
     def get_comp_end(self):
         '''
@@ -814,7 +985,11 @@ class ProvPopTable(pd.DataFrame):
         result.append(self.__comp_neg_emi)
         result.append(self.__comp_neg_npr_out)
         result.append(self.__comp_neg_death)
+        if self.__comp_neg_put_etc_before:
+            result.extend(self.__comp_neg_etc)
         result.append(self.__comp_neg_interprov_out)
+        if not self.__comp_neg_put_etc_before:
+            result.extend(self.__comp_neg_etc)
         for comp in result.copy():
             if comp is None:
                 result.remove(comp)
@@ -835,7 +1010,11 @@ class ProvPopTable(pd.DataFrame):
         result.append(self.__comp_pos_ret_emi)
         result.append(self.__comp_pos_npr_in)
         result.append(self.__comp_pos_immi)
+        if self.__comp_pos_put_etc_before:
+            result.extend(self.__comp_pos_etc)
         result.append(self.__comp_pos_interprov_in)
+        if not self.__comp_pos_put_etc_before:
+            result.extend(self.__comp_pos_etc)
         for comp in result.copy():
             if comp is None:
                 result.remove(comp)
@@ -1491,7 +1670,7 @@ def calculate_modifiable_in_comp(
     else:
         raise NotImplementedError
 
-    df_comp_modifiable["{0}_J".format(comp)] = modifiable
+    df_comp_modifiable[f"{comp}_J"] = modifiable
     del df_comp_modifiable[comp]
 
     return df_comp_modifiable
