@@ -15,10 +15,32 @@ from estime2 import (
 )
 from estime2.provpoptable import calculate_ages_to_modify_and_counter
 from pprint import pprint
+import estime2
 import numpy as np
 import os
 import pandas as pd
-pd.options.display.max_rows = 15
+pd.options.display.min_rows = 15
+
+RegionSAD = ['RegionCode', 'Sex', 'Age', 'RefDate']
+RegionSA = RegionSAD[0:3]
+RegionSD = list(np.array(RegionSAD)[[0,1,3]])
+RegionS = RegionSA[0:2]
+    
+def arrange_by_lvl(df, levels, value_name):
+    age_cols = [col for col in df if col.startswith('A_')]
+    result =\
+        pd.melt(
+            df[levels + age_cols],
+            id_vars = levels,
+            value_vars = age_cols,
+            var_name = 'Age',
+            value_name = value_name
+        )\
+        .assign(
+            Age = lambda df: \
+                df.Age.apply(lambda x: int(x[(x.index('A_') + 2):]))
+        )
+    return result
 
 def call_tbl():
     files_above = os.listdir('..')
@@ -28,6 +50,42 @@ def call_tbl():
     poptbl = ProvPopTable(tbl)
     return poptbl
 
+def call_tbl2():
+    files_above = os.listdir('..')
+    starts_with_10 = list(map(lambda x: x[0:2] == '10', files_above))
+    tbl_name = np.array(files_above)[starts_with_10][0]
+
+    with pd.ExcelFile('../' + tbl_name) as tbl:
+        pop_2001 = pd.read_excel(tbl, sheet_name = 0)
+        bth = pd.read_excel(tbl, sheet_name = 1)
+        dth_d01 = pd.read_excel(tbl, sheet_name = 2)
+        dth_d02 = pd.read_excel(tbl, sheet_name = 3)
+        imm = pd.read_excel(tbl, sheet_name = 4)
+        emi = pd.read_excel(tbl, sheet_name = 5)
+        rem = pd.read_excel(tbl, sheet_name = 6)
+        nte = pd.read_excel(tbl, sheet_name = 7)
+        mip = pd.read_excel(tbl, sheet_name = 8)
+        mit = pd.read_excel(tbl, sheet_name = 9)
+        npr = pd.read_excel(tbl, sheet_name = 10)
+        pop_2002_orig = pd.read_excel(tbl, sheet_name = 11, skiprows = 2)
+        pop_2002_corrected = pd.read_excel(tbl, sheet_name = 12)
+
+    return {
+        'pop_2001': pop_2001, 
+        'bth': bth,
+        'dth_d01': dth_d01,
+        'dth_d02': dth_d02,
+        'imm': imm,
+        'emi': emi,
+        'rem': rem,
+        'nte': nte,
+        'mip': mip,
+        'mit': mit,
+        'npr': npr,
+        'pop_2002_orig': pop_2002_orig,
+        'pop_2002_corrected': pop_2002_corrected
+    }
+
 def run():
     poptbl = call_tbl()
 
@@ -650,3 +708,139 @@ def run4():
         )
     
     return df_comp_counter_adjust_reversed
+
+def run5():
+    result = call_tbl2()
+
+    pop_2001 = result['pop_2001']
+    bth = result['bth']
+    dth_d01 = result['dth_d01']
+    dth_d02 = result['dth_d02']
+    imm = result['imm']
+    emi = result['emi']
+    rem = result['rem']
+    nte = result['nte']
+    mip = result['mip']
+    mit = result['mit']
+    npr = result['npr']
+    pop_2002_orig = result['pop_2002_orig']
+    pop_2002_corrected = result['pop_2002_corrected']
+    
+    nprs = arrange_by_lvl(npr, RegionSD, 'NPR')
+    tbl =\
+        arrange_by_lvl(pop_2001, RegionS, 'InitialPopulation')\
+        .merge(
+            bth[RegionS + ['Total']]\
+                .assign(Age = -1)\
+                .rename(columns = {'Total': 'BTH'})\
+                [RegionSA + ['BTH']],
+            on = RegionSA,
+            how = 'outer'
+        )\
+        .sort_values(RegionSA)\
+        .fillna(0)\
+        .assign(
+            InitialPopulation = lambda df: df.InitialPopulation.apply(int),
+            BTH = lambda df: df.BTH.apply(int)
+        )\
+        .rename(
+            columns = {'InitialPopulation': 'POP_2001'}
+        )\
+        .merge(
+            arrange_by_lvl(dth_d01, RegionS, 'DTH_D01'),
+            on = RegionSA,
+            how = 'inner'
+        )\
+        .merge(
+            arrange_by_lvl(imm, RegionS, 'IMM'),
+            on = RegionSA,
+            how = 'inner'
+        )\
+        .merge(
+            arrange_by_lvl(emi, RegionS, 'EMI'),
+            on = RegionSA,
+            how = 'inner'
+        )\
+        .merge(
+            arrange_by_lvl(rem, RegionS, 'REM'),
+            on = RegionSA,
+            how = 'inner'
+        )\
+        .merge(
+            arrange_by_lvl(nte, RegionS, 'NTE'),
+            on = RegionSA,
+            how = 'inner'
+        )\
+        .merge(
+            arrange_by_lvl(mip, RegionS, 'MIP'),
+            on = RegionSA,
+            how = 'inner'
+        )\
+        .merge(
+            arrange_by_lvl(mit, RegionS, 'MIT'),
+            on = RegionSA,
+            how = 'inner'
+        )\
+        .merge(
+            nprs.loc[
+                lambda df: df['RefDate'] < np.datetime64('2002-01-01')
+            ]\
+            [RegionSA + ['NPR']]\
+            .rename(columns = {'NPR': 'NPR_out'}),
+            on = RegionSA,
+            how = 'outer'
+        )\
+        .merge(
+            nprs.loc[
+                lambda df: df['RefDate'] > np.datetime64('2002-01-01')
+            ]\
+            [RegionSA + ['NPR']]\
+            .rename(columns = {'NPR': 'NPR_in'}),
+            on = RegionSA,
+            how = 'outer'
+        )\
+        .fillna(0)\
+        .assign(
+            NPR_out = lambda df: df.NPR_out.apply(int),
+            NPR_in = lambda df: df.NPR_in.apply(int)
+        )\
+        .assign(
+            MIP_out = lambda df: df.MIP.apply(lambda x: abs(min(x, 0))),
+            MIP_in = lambda df: df.MIP.apply(lambda x: max(x, 0)),
+            MIT_out = lambda df: df.MIT.apply(lambda x: abs(min(x, 0))),    
+            MIT_in = lambda df: df.MIT.apply(lambda x: max(x, 0))
+        )
+    selected_cols = tbl.columns.tolist()
+    selected_cols.remove('MIP')
+    selected_cols.remove('MIT')
+    tbl = tbl[selected_cols]
+    
+    all_RegionC = np.unique(tbl[RegionSAD[0]])
+    poptbl_regions = {}
+    for region in all_RegionC:
+        tbl_region =\
+            tbl\
+            .query(f"{RegionSAD[0]} == {region}")\
+            [selected_cols[1:]]
+        with estime2.option_context(
+            'pop.sex', 'Sex',
+            'pop.age', 'Age',
+            'pop.end', 'POP_2002',
+            'pop.start', 'POP_2001',
+            'pop.birth', 'BTH',
+            'comp_neg.temp_out', 'NTE',
+            'comp_neg.emi', 'EMI',
+            'comp_neg.npr_out', 'NPR_out',
+            'comp_neg.death', 'DTH_D01',
+            'comp_neg.interprov_out', 'MIP_out',
+            'comp_neg.etc', ['MIT_out'],
+            'comp_pos.ret_emi', 'REM',
+            'comp_pos.npr_in', 'NPR_in',
+            'comp_pos.immi', 'IMM',
+            'comp_pos.interprov_in', 'MIP_in',
+            'comp_pos.etc', ['MIT_in'],
+            'comp.end', ['NPR_in']
+        ):
+            poptbl_regions[region] = estime2.ProvPopTable(tbl_region)
+    
+    return poptbl_regions
\ No newline at end of file
